@@ -13,8 +13,8 @@ from impira import Impira as ImpiraAPI, FieldType, InferredFieldType, parse_date
 from ..cmd.utils import environ_or_required
 from ..schema import schema_to_model
 from ..types import (
-    BBox,
-    combine_bboxes,
+    Location,
+    combine_locations,
     CheckboxLabel,
     SignatureLabel,
     DocData,
@@ -71,7 +71,7 @@ def is_overlapping(left1, width1, left2, width2):
 
 
 @validate_arguments
-def is_bbox_overlapping(bbox1: BBox, bbox2: BBox):
+def is_bbox_overlapping(bbox1: Location, bbox2: Location):
     return (
         bbox1.page == bbox2.page
         and is_overlapping(
@@ -86,7 +86,7 @@ def is_bbox_overlapping(bbox1: BBox, bbox2: BBox):
 
 class ImpiraWord(BaseModel):
     confidence: float
-    location: BBox
+    location: Location
     processed_word: str
     rotated: bool
     source: str
@@ -96,7 +96,7 @@ class ImpiraWord(BaseModel):
 
 class ImpiraEntity(BaseModel):
     label: str
-    location: BBox
+    location: Location
     processed: Any
     source_rivlets: List[str]
     uid: str
@@ -112,7 +112,7 @@ FirstClassEntityLabelToFieldType = {
 
 
 class CheckboxSource(BaseModel):
-    BBoxes: List[BBox]
+    BBoxes: List[Location]
 
 
 class ScalarLabel(BaseModel):
@@ -234,13 +234,13 @@ def generate_labels(
                 ),
                 ModelVersion=model_versions.get(field_name, 0),
             )
-        elif isinstance(value, CheckboxLabel) or isinstance(value, SignatureLabel):
+        elif isinstance(value, (CheckboxLabel, SignatureLabel)):
             scalar_label = ScalarLabel(
                 Label=ScalarLabel.L(
-                    Source=CheckboxSource(BBoxes=[value.location]),
+                    Source=CheckboxSource(BBoxes=[value.location] if value.location else []),
                     Value={
-                        "Value": value.value,
-                        "State": 1 if value.value else 0,
+                        "Value": value.value == 1 if value.value is not None else None,
+                        "State": value.value,
                     },
                 ),
                 Context=ScalarLabel.C(Entities=[]),
@@ -369,33 +369,59 @@ def fields_to_doc_schema(fields) -> DocSchema:
     return DocSchema(fields=ret)
 
 
+@validate_arguments
+def maybe_get_location(label: ScalarLabel, field_type: str) -> Optional[Location]:
+
+    if label.Label.Source is None:
+        return None
+
+    if field_type in ("CheckboxLabel", "SignatureLabel"):
+        locations = label.Label.Source.BBoxes
+    elif field_type in ("TextLabel", "NumberLabel", "TimestampLabel"):
+        locations = [l.location for l in label.Label.Source]
+    elif field_type == "DocumentTagLabel":
+        locations = None
+    else:
+        raise ValueError(f"Unable to retrieve location from label of type {field_type} ({label = })")
+
+    if locations:
+        return combine_locations(locations)
+    return None
+
+
+@validate_arguments
+def maybe_get_value(label: ScalarLabel, field_type: str) -> Optional[Any]:
+
+    value = label.Label.Value
+    if field_type in ("CheckboxLabel", "SignatureLabel"):
+        value = value["State"]
+    elif field_type == "TimestampLabel" and value is not None:
+        value = parse_date(value)
+
+    return value
+
+
 @validate_arguments
 def row_to_record(row, doc_schema: DocSchema) -> Any:
     d = {}
     for field_name, field_type in doc_schema.fields.items():
         label = None
-        value = row.get(field_name, None)
+        field = row.get(field_name, None)
         if isinstance(field_type, DocSchema):
-            table_rows = [row_label["Label"]["Value"] for row_label in value["Label"]["Value"]]
+            table_rows = [row_label["Label"]["Value"] for row_label in field["Label"]["Value"]]
             label = [x for x in [row_to_record(tr, field_type) for tr in table_rows] if x is not None]
             if not label:
                 continue
-        elif value is not None:
-            impira_label = ScalarLabel.parse_obj(value)
+        elif field is not None:
+            impira_label = ScalarLabel.parse_obj(field)
             if impira_label.Label.IsPrediction:
                 continue
-            location = (
-                combine_bboxes(*[x.location for x in impira_label.Label.Source])
-                if len(impira_label.Label.Source) > 0
-                else None
-            )
-            scalar = impira_label.Label.Value
-            if scalar is not None:
-                if field_type == "TimestampLabel":
-                    scalar = parse_date(scalar)
-                elif field_type == "CheckboxLabel" or field_type == "SignatureLabel":
-                    scalar = scalar["Value"]  # Checkboxes are nested inside of an extra 'Value'
-            label = {"location": location, "value": scalar}
+
+            location = maybe_get_location(impira_label, field_type)
+            value = maybe_get_value(impira_label, field_type)
+
+            label = {"location": location, "value": value}
+
         d[field_name] = label
 
     if len(d) == 0:
@@ -592,22 +618,36 @@ class Impira(Tool):
         field_names_to_update = set()
         for f in schema[:max_fields] if max_fields != -1 else schema:
             field_type = f.field_type
-            first_labels = labels[0]
-            if f.name in first_labels and isinstance(first_labels[f.name], ScalarLabel) and not skip_type_inference:
-                entities = first_labels[f.name].Context.Entities
-                unique_entity_types = set(
-                    [field_type]
-                    + [
-                        FirstClassEntityLabelToFieldType[e.label]
-                        for e in entities
-                        if e.label in FirstClassEntityLabelToFieldType
-                    ]
-                )
 
-                if InferredFieldType.timestamp in unique_entity_types:
-                    field_type = InferredFieldType.timestamp
-                elif InferredFieldType.number in unique_entity_types:
-                    field_type = InferredFieldType.number
+            if not skip_type_inference:
+                narrow_types = set()
+                for label in labels:
+                    if f.name in label and isinstance(label[f.name], ScalarLabel):
+                        entities = label[f.name].Context.Entities
+                        unique_entity_types = set(
+                            [field_type]
+                            + [
+                                FirstClassEntityLabelToFieldType[e.label]
+                                for e in entities
+                                if e.label in FirstClassEntityLabelToFieldType
+                            ]
+                        )
+
+                    if InferredFieldType.timestamp in unique_entity_types:
+                        narrow_type = InferredFieldType.timestamp
+                    elif InferredFieldType.number in unique_entity_types:
+                        narrow_type = InferredFieldType.number
+                    else:
+                        narrow_type = field_type
+
+                    narrow_types.add(narrow_type)
+                    if narrow_type == InferredFieldType.text:
+                        break
+
+                for weak_type in [InferredFieldType.text, InferredFieldType.number, InferredFieldType.timestamp]:
+                    if weak_type in narrow_types:
+                        field_type = weak_type
+                        break
 
             if f.name in current_fields or (len(f.path) > 0 and f.path[0] in current_fields):
                 existing_field = current_fields.get(f.name) or current_fields.get(f.path[0])
@@ -708,7 +748,7 @@ class Impira(Tool):
                     )
                     break
                 except APIError as e:
-                    log.warning("Failed to update. Will retry up to %d more times" % (10 - i - 1))
+                    log.warning("Failed to update. Will retry up to %d more times: %s" % (10 - i - 1, e))
 
         log.info("Done running update on %d files. Models will now update!" % len(labeled_files))
 
@@ -735,11 +775,16 @@ class Impira(Tool):
         log.info("Done!")
 
     @validate_arguments
-    def snapshot(self, collection_uid: str, use_original_filenames=False, labeled_files_only=False):
+    def snapshot(
+        self, collection_uid: str, use_original_filenames=False, labeled_files_only=False, filter_collection_uid=None
+    ):
         log = self._log()
 
         conn = self._conn()
-        resp = conn.query("@file_collections::%s" % (collection_uid))
+
+        filt = f"-join_one(`file_collections::{filter_collection_uid}`, uid, uid)=null" if filter_collection_uid else ""
+
+        resp = conn.query("@`file_collections::%s` %s" % (collection_uid, filt))
 
         doc_schema = fields_to_doc_schema(filter_inferred_fields(resp["schema"]["children"]))
         records = [
@@ -763,7 +808,9 @@ class Impira(Tool):
         log = self._log()
 
         conn = self._conn()
-        files = conn.query("@files[uid, File: File[download_url, name]] -File.download_url=null -`File type`=Data")["data"]
+        files = conn.query("@files[uid, File: File[download_url, name]] -File.download_url=null -`File type`=Data")[
+            "data"
+        ]
         collections = conn.query(
             "@file_collection_contents[collection_uid, files: array_agg(file_uid)] -collection=null"
         )["data"]
